div.content[data-v-22613268]{display:flex}.tabs{display:flex;flex-direction:row;border-bottom:1px solid #e9e9e9}.tabs .tab{padding:10px 20px 10px 0;cursor:pointer;font-weight:300}.selected:is(.tabs .tab){border-bottom:solid 1px black;font-weight:400}main[data-v-36865594]{padding:0 5vw;font-size:12px}.dataframe[data-v-36865594]{margin-top:10px}.images[data-v-36865594]{display:grid;grid-template-columns:1fr 1fr 1fr 1fr;place-items:center center}.buttons>p[data-v-36865594]{padding:10px}.toasts[data-v-36865594]{padding-top:10px}.toasts[data-v-36865594]>*{margin-bottom:10px}.text-inputs>p[data-v-36865594]{padding:10px}.editable-list-tab[data-v-36865594]{display:grid;padding-top:10px;grid-template-columns:1fr 1fr}.editable-list-container[data-v-36865594]{display:block;width:33vw;height:300px}.editable-list-container .editable-list-container-scrollable[data-v-36865594]{height:100%;overflow-y:auto}.icons[data-v-36865594]{display:grid;padding-top:10px;gap:20px;grid-template-columns:1fr 1fr 1fr 1fr}.icons [class^=icon-][data-v-36865594],.icons [class*=" icon-"][data-v-36865594]{color:var(--color-primary);font-size:20px}
